@@ -9,11 +9,11 @@ Test::WWW::Mechanize - Testing-specific WWW::Mechanize subclass
 
 =head1 VERSION
 
-Version 1.23_01
+Version 1.31_01
 
 =cut
 
-our $VERSION = '1.23_01';
+our $VERSION = '1.31_01';
 
 =head1 SYNOPSIS
 
@@ -26,9 +26,9 @@ features for web application testing.  For example:
     my $mech = Test::WWW::Mechanize->new;
     $mech->get_ok( $page );
     $mech->base_is( 'http://petdance.com/', 'Proper <BASE HREF>' );
-    $mech->title_is( "Invoice Status", "Make sure we're on the invoice page" );
-    $mech->content_contains( "Andy Lester", "My name somewhere" );
-    $mech->content_like( qr/(cpan|perl)\.org/, "Link to perl.org or CPAN" );
+    $mech->title_is( 'Invoice Status', "Make sure we're on the invoice page" );
+    $mech->text_contains( 'Andy Lester', 'My name somewhere' );
+    $mech->content_like( qr/(cpan|perl)\.org/, 'Link to perl.org or CPAN' );
 
 This is equivalent to:
 
@@ -39,9 +39,9 @@ This is equivalent to:
     $mech->get( $page );
     ok( $mech->success );
     is( $mech->base, 'http://petdance.com', 'Proper <BASE HREF>' );
-    is( $mech->title, "Invoice Status", "Make sure we're on the invoice page" );
-    ok( index( $mech->content, "Andy Lester" ) >= 0, "My name somewhere" );
-    like( $mech->content, qr/(cpan|perl)\.org/, "Link to perl.org or CPAN" );
+    is( $mech->title, 'Invoice Status', "Make sure we're on the invoice page" );
+    ok( index( $mech->content( format => 'text' ), 'Andy Lester' ) >= 0, 'My name somewhere' );
+    like( $mech->content, qr/(cpan|perl)\.org/, 'Link to perl.org or CPAN' );
 
 but has nicer diagnostics if they fail.
 
@@ -50,8 +50,8 @@ Default descriptions will be supplied for most methods if you omit them. e.g.
     my $mech = Test::WWW::Mechanize->new;
     $mech->get_ok( 'http://petdance.com/' );
     $mech->base_is( 'http://petdance.com/' );
-    $mech->title_is( "Invoice Status" );
-    $mech->content_contains( "Andy Lester" );
+    $mech->title_is( 'Invoice Status' );
+    $mech->content_contains( 'Andy Lester' );
     $mech->content_like( qr/(cpan|perl)\.org/ );
 
 results in
@@ -59,7 +59,7 @@ results in
     ok - Got 'http://petdance.com/' ok
     ok - Base is 'http://petdance.com/'
     ok - Title is 'Invoice Status'
-    ok - Content contains 'Andy Lester'
+    ok - Text contains 'Andy Lester'
     ok - Content is like '(?-xism:(cpan|perl)\.org)'
 
 =cut
@@ -90,16 +90,8 @@ called.
 
 =item * get_ok()
 
-=back
-
-and will eventually do the same after any of the following:
-
-=over
-
 =item * post_ok()
 
-=item * back_ok()
-
 =item * submit_form_ok()
 
 =item * follow_link_ok()
@@ -108,7 +100,7 @@ and will eventually do the same after any of the following:
 
 =back
 
-This means you no longerhave to do the following:
+This means you no longer have to do the following:
 
     my $mech = Test::WWW::Mechanize->new();
     $mech->get_ok( $url, 'Fetch the intro page' );
@@ -156,38 +148,12 @@ A default description of "GET $url" is used if none if provided.
 
 sub get_ok {
     my $self = shift;
-    my $url = shift;
 
-    my $desc;
-    my %opts;
-
-    if ( @_ ) {
-        my $flex = shift; # The flexible argument
-
-        if ( !defined( $flex ) ) {
-            $desc = shift;
-        }
-        elsif ( ref $flex eq 'HASH' ) {
-            %opts = %{$flex};
-            $desc = shift;
-        }
-        elsif ( ref $flex eq 'ARRAY' ) {
-            %opts = @{$flex};
-            $desc = shift;
-        }
-        else {
-            $desc = $flex;
-        }
-    } # parms left
+    my ($url,$desc,%opts) = $self->_unpack_args( 'GET', @_ );
 
     $self->get( $url, %opts );
     my $ok = $self->success;
 
-    if ( not defined $desc ) {
-        $url = $url->url if ref($url) eq 'WWW::Mechanize::Link';
-        $desc = "GET $url";
-    }
-
     $ok = $self->_maybe_lint( $ok, $desc );
 
     return $ok;
@@ -230,37 +196,12 @@ A default description of "HEAD $url" is used if none if provided.
 
 sub head_ok {
     my $self = shift;
-    my $url = shift;
 
-    my $desc;
-    my %opts;
-
-    if ( @_ ) {
-        my $flex = shift; # The flexible argument
-
-        if ( !defined( $flex ) ) {
-            $desc = shift;
-        }
-        elsif ( ref $flex eq 'HASH' ) {
-            %opts = %{$flex};
-            $desc = shift;
-        }
-       elsif ( ref $flex eq 'ARRAY' ) {
-            %opts = @{$flex};
-            $desc = shift;
-        }
-        else {
-            $desc = $flex;
-        }
-    } # parms left
+    my ($url,$desc,%opts) = $self->_unpack_args( 'HEAD', @_ );
 
     $self->head( $url, %opts );
     my $ok = $self->success;
 
-    if ( not defined $desc ) {
-        $url = $url->url if ref($url) eq 'WWW::Mechanize::Link';
-        $desc = "HEAD $url";
-    }
     $Test->ok( $ok, $desc );
     if ( !$ok ) {
         $Test->diag( $self->status );
@@ -270,6 +211,7 @@ sub head_ok {
     return $ok;
 }
 
+
 =head2 $mech->post_ok( $url, [ \%LWP_options ,] $desc )
 
 A wrapper around WWW::Mechanize's post(), with similar options, except
@@ -283,41 +225,12 @@ A default description of "POST to $url" is used if none if provided.
 
 sub post_ok {
     my $self = shift;
-    my $url = shift;
 
-    my $desc;
-    my %opts;
+    my ($url,$desc,%opts) = $self->_unpack_args( 'POST', @_ );
 
-    if ( @_ ) {
-        my $flex = shift; # The flexible argument
-
-        if ( !defined( $flex ) ) {
-            $desc = shift;
-        }
-        elsif ( ref $flex eq 'HASH' ) {
-            %opts = %{$flex};
-            $desc = shift;
-        }
-        elsif ( ref $flex eq 'ARRAY' ) {
-            %opts = @{$flex};
-            $desc = shift;
-        }
-        else {
-            $desc = $flex;
-        }
-    } # parms left
-
-    if ( not defined $desc ) {
-        $url = $url->url if ref($url) eq 'WWW::Mechanize::Link';
-        $desc = "POST $url";
-    }
     $self->post( $url, \%opts );
     my $ok = $self->success;
-    $Test->ok( $ok, $desc );
-    if ( !$ok ) {
-        $Test->diag( $self->status );
-        $Test->diag( $self->response->message ) if $self->response;
-    }
+    $ok = $self->_maybe_lint( $ok, $desc );
 
     return $ok;
 }
@@ -335,35 +248,11 @@ A default description of "PUT to $url" is used if none if provided.
 
 sub put_ok {
     my $self = shift;
-    my $url = shift;
 
-    my $desc;
-    my %opts;
+    my ($url,$desc,%opts) = $self->_unpack_args( 'PUT', @_ );
+    $opts{content} = '' if !exists $opts{content};
+    $self->put( $url, %opts );
 
-    if ( @_ ) {
-        my $flex = shift; # The flexible argument
-
-        if ( !defined( $flex ) ) {
-            $desc = shift;
-        }
-        elsif ( ref $flex eq 'HASH' ) {
-            %opts = %{$flex};
-            $desc = shift;
-        }
-        elsif ( ref $flex eq 'ARRAY' ) {
-            %opts = @{$flex};
-            $desc = shift;
-        }
-        else {
-            $desc = $flex;
-        }
-    } # parms left
-
-    if ( not defined $desc ) {
-        $url = $url->url if ref($url) eq 'WWW::Mechanize::Link';
-        $desc = "PUT $url";
-    }
-    $self->put( $url, \%opts );
     my $ok = $self->success;
     $Test->ok( $ok, $desc );
     if ( !$ok ) {
@@ -384,7 +273,13 @@ I<%parms> is a hashref containing the parms to pass to C<submit_form()>.
 Note that the parms to C<submit_form()> are a hash whereas the parms to
 this function are a hashref.  You have to call this function like:
 
-    $agent->submit_form_ok( {n=>3}, "looking for 3rd link" );
+    $mech->submit_form_ok( {
+            form_number => 3,
+            fields      => {
+                answer => 42
+            },
+        }, 'now we just need the question'
+    );
 
 As with other test functions, C<$desc> is optional.  If it is supplied
 then it will display when running the test harness in verbose mode.
@@ -421,8 +316,7 @@ sub submit_form_ok {
         }
     }
 
-    $Test->ok( $ok, $desc );
-    $Test->diag( $error ) if $error;
+    $ok = $self->_maybe_lint( $ok, $desc );
 
     return $ok;
 }
@@ -438,7 +332,7 @@ I<%parms> is a hashref containing the parms to pass to C<follow_link()>.
 Note that the parms to C<follow_link()> are a hash whereas the parms to
 this function are a hashref.  You have to call this function like:
 
-    $mech->follow_link_ok( {n=>3}, "looking for 3rd link" );
+    $mech->follow_link_ok( {n=>3}, 'looking for 3rd link' );
 
 As with other test functions, C<$desc> is optional.  If it is supplied
 then it will display when running the test harness in verbose mode.
@@ -480,8 +374,7 @@ sub follow_link_ok {
         }
     }
 
-    $Test->ok( $ok, $desc );
-    $Test->diag( $error ) if $error;
+    $ok = $self->_maybe_lint( $ok, $desc );
 
     return $ok;
 }
@@ -504,14 +397,52 @@ sub click_ok {
         return $Test->ok( 0, $desc );
     }
 
-    if ( !$response->is_success ) {
-        $Test->diag( "Failed test $desc:" );
-        $Test->diag( $response->as_string );
-        return $Test->ok( 0, $desc );
-    }
-    return $Test->ok( 1, $desc );
+
+    my $ok = $response->is_success;
+
+    $ok = $self->_maybe_lint( $ok, $desc );
+
+    return $ok;
 }
 
+
+sub _unpack_args {
+    my $self   = shift;
+    my $method = shift;
+    my $url    = shift;
+
+    my $desc;
+    my %opts;
+
+    if ( @_ ) {
+        my $flex = shift; # The flexible argument
+
+        if ( !defined( $flex ) ) {
+            $desc = shift;
+        }
+        elsif ( ref $flex eq 'HASH' ) {
+            %opts = %{$flex};
+            $desc = shift;
+        }
+        elsif ( ref $flex eq 'ARRAY' ) {
+            %opts = @{$flex};
+            $desc = shift;
+        }
+        else {
+            $desc = $flex;
+        }
+    } # parms left
+
+    if ( not defined $desc ) {
+        $url = $url->url if ref($url) eq 'WWW::Mechanize::Link';
+        $desc = "$method $url";
+    }
+
+    return ($url, $desc, %opts);
+}
+
+
+
 =head1 METHODS: CONTENT CHECKING
 
 =head2 $mech->html_lint_ok( [$desc] )
@@ -579,7 +510,7 @@ sub _lint_content_ok {
 
 Tells if the title of the page is the given string.
 
-    $mech->title_is( "Invoice Summary" );
+    $mech->title_is( 'Invoice Summary' );
 
 =cut
 
@@ -633,7 +564,7 @@ sub title_unlike {
 
 Tells if the base of the page is the given string.
 
-    $mech->base_is( "http://example.com/" );
+    $mech->base_is( 'http://example.com/' );
 
 =cut
 
@@ -763,15 +694,74 @@ Tells if the content of the page does NOT match I<$regex>.
 =cut
 
 sub content_unlike {
-    my $self = shift;
+    my $self  = shift;
     my $regex = shift;
-    my $desc = shift;
-    $desc = qq{Content is like "$regex"} if !defined($desc);
+    my $desc  = shift || qq{Content is unlike "$regex"};
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     return unlike_string( $self->content, $regex, $desc );
 }
 
+=head2 $mech->text_contains( $str [, $desc ] )
+
+Tells if the text form of the page's content contains I<$str>.
+
+When your page contains HTML which is difficult, unimportant, or
+unlikely to match over time as designers alter markup, use
+C<text_contains> instead of L</content_contains>.
+
+ # <b>Hi, <i><a href="some/path">User</a></i>!</b>
+ $mech->content_contains('Hi, User'); # Fails.
+ $mech->text_contains('Hi, User'); # Passes.
+
+Text is determined by calling C<< $mech->content(format => 'text') >>.
+See L<WWW::Mechanize/content>.
+
+=cut
+
+sub text_contains {
+    my $self = shift;
+    my $str  = shift;
+    my $desc = shift || qq{Text contains "$str"};
+
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+    if ( ref($str) eq 'REGEX' ) {
+        diag( 'text_contains takes a string, not a regex' );
+    }
+
+    return contains_string( $self->content(format => "text"), $str, $desc );
+}
+
+=head2 $mech->text_like( $regex [, $desc ] )
+
+Tells if the text form of the page's content matches I<$regex>.
+
+=cut
+
+sub text_like {
+    my $self  = shift;
+    my $regex = shift;
+    my $desc  = shift || qq{Text is like "$regex"};
+
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+    return like_string( $self->text, $regex, $desc );
+}
+
+=head2 $mech->text_unlike( $regex [, $desc ] )
+
+Tells if the text format of the page's content does NOT match I<$regex>.
+
+=cut
+
+sub text_unlike {
+    my $self  = shift;
+    my $regex = shift;
+    my $desc  = shift || qq{Text is unlike "$regex"};
+
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+    return unlike_string( $self->text, $regex, $desc );
+}
+
 =head2 $mech->has_tag( $tag, $text [, $desc ] )
 
 Tells if the page has a C<$tag> tag with the given content in its text.
@@ -782,8 +772,7 @@ sub has_tag {
     my $self = shift;
     my $tag  = shift;
     my $text = shift;
-    my $desc = shift;
-    $desc = qq{Page has $tag tag with "$text"} if !defined($desc);
+    my $desc = shift || qq{Page has $tag tag with "$text"};
 
     my $found = $self->_tag_walk( $tag, sub { $text eq $_[0] } );
 
@@ -1054,7 +1043,7 @@ sub link_content_like {
     }
 
     my @urls = _format_links( $links );
-    $desc = _default_links_desc(\@urls, "are like '$regex'") if !defined($desc);
+    $desc = _default_links_desc( \@urls, qq{are like "$regex"} ) if !defined($desc);
     my @failures = $self->_check_links_content( \@urls, $regex );
     my $ok = (@failures == 0);
 
@@ -1091,7 +1080,7 @@ sub link_content_unlike {
     }
 
     my @urls = _format_links( $links );
-    $desc = _default_links_desc(\@urls, qq{are not like "$regex"}) if !defined($desc);
+    $desc = _default_links_desc( \@urls, qq{are not like "$regex"} ) if !defined($desc);
     my @failures = $self->_check_links_content( \@urls, $regex, 'unlike' );
     my $ok = (@failures == 0);
 
@@ -1380,6 +1369,8 @@ L<http://search.cpan.org/dist/Test-WWW-Mechanize>
 =head1 ACKNOWLEDGEMENTS
 
 Thanks to
+Philip G. Potter,
+Niko Tyni,
 Greg Sheard,
 Michael Schwern,
 Mark Blackman,
@@ -1392,10 +1383,20 @@ and Pete Krawczyk for patches.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2004-2008 Andy Lester, all rights reserved.
+Copyright 2004-2011 Andy Lester.
 
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
+This program is free software; you can redistribute it and/or
+modify it under the terms of either:
+
+=over 4
+
+=item * the GNU General Public License as published by the Free
+Software Foundation; either version 1, or (at your option) any
+later version, or
+
+=item * the Artistic License version 2.0.
+
+=back
 
 =cut
 
